@@ -1,6 +1,6 @@
 from .version import ELECTRUM_VERSION
 from .util import format_satoshis, print_msg, print_error, set_verbosity
-from .wallet import Synchronizer, Wallet
+from .wallet import Wallet
 from .storage import WalletStorage
 from .coinchooser import COIN_CHOOSERS
 from .network import Network, pick_random_server
@@ -10,5 +10,5 @@ from . import bitcoin
 from . import transaction
 from . import daemon
 from .transaction import Transaction
-from .plugins import BasePlugin
+from .plugin import BasePlugin
 from .commands import Commands, known_commands
