@@ -28,8 +28,9 @@ import os
 
 import requests
 
-from electrum import ELECTRUM_VERSION, constants
-from electrum.i18n import _
+from .version import ELECTRUM_VERSION
+from .import constants
+from .i18n import _
 
 
 class BaseCrashReporter(object):
@@ -43,6 +44,7 @@ class BaseCrashReporter(object):
 <h2>Additional information</h2>
 <ul>
   <li>Electrum version: {app_version}</li>
+  <li>Python version: {python_version}</li>
   <li>Operating system: {os}</li>
   <li>Wallet type: {wallet_type}</li>
   <li>Locale: {locale}</li>
@@ -86,6 +88,7 @@ class BaseCrashReporter(object):
     def get_additional_info(self):
         args = {
             "app_version": ELECTRUM_VERSION,
+            "python_version": sys.version,
             "os": self.get_os_version(),
             "wallet_type": "unknown",
             "locale": locale.getdefaultlocale()[0] or "?",
