@@ -12,7 +12,7 @@ from decimal import Decimal
 
 from .bitcoin import COIN
 from .i18n import _
-from .util import PrintError, ThreadJob
+from .util import PrintError, ThreadJob, make_dir
 
 
 # See https://en.wikipedia.org/wiki/ISO_4217
@@ -114,7 +114,6 @@ class ExchangeBase(PrintError):
         rates = self.get_rates('')
         return sorted([str(a) for (a, b) in rates.items() if b is not None and len(a)==3])
 
-
 class BitcoinAverage(ExchangeBase):
 
     def get_rates(self, ccy):
@@ -325,9 +324,12 @@ class NegocieCoins(ExchangeBase):
         json = self.get_json('api.bitvalor.com', '/v1/ticker.json')
         return {'BRL': Decimal(json['ticker_1h']['exchanges']['NEG']['last'])}
 
-    def history_ccys(self):
-        return ['BRL']
+class TheRockTrading(ExchangeBase):
 
+    def get_rates(self, ccy):
+        json = self.get_json('api.therocktrading.com', 
+                             '/v1/funds/BTCEUR/ticker')
+        return {'EUR': Decimal(json['last'])}
 
 class Unocoin(ExchangeBase):
 
@@ -430,8 +432,7 @@ class FxThread(ThreadJob):
         self.hist_checkbox = None
         self.cache_dir = os.path.join(config.path, 'cache')
         self.set_exchange(self.config_exchange())
-        if not os.path.exists(self.cache_dir):
-            os.mkdir(self.cache_dir)
+        make_dir(self.cache_dir)
 
     def get_currencies(self, h):
         d = get_exchanges_by_ccy(h)
@@ -567,6 +568,6 @@ class FxThread(ThreadJob):
         return self.fiat_value(satoshis, self.history_rate(d_t))
 
     def timestamp_rate(self, timestamp):
-        from electrum.util import timestamp_to_datetime
+        from .util import timestamp_to_datetime
         date = timestamp_to_datetime(timestamp)
         return self.history_rate(date)
