@@ -26,15 +26,14 @@
 
 from unicodedata import normalize
 
-from . import bitcoin, ecc
+from . import bitcoin, ecc, constants
 from .bitcoin import *
 from .ecc import string_to_number, number_to_string
 from .crypto import pw_decode, pw_encode
-from . import constants
 from .util import (PrintError, InvalidPassword, hfu, WalletFileException,
                    BitcoinException)
 from .mnemonic import Mnemonic, load_wordlist
-from .plugins import run_hook
+from .plugin import run_hook
 
 
 class KeyStore(PrintError):
@@ -553,13 +552,12 @@ def bip39_normalize_passphrase(passphrase):
     return normalize('NFKD', passphrase or '')
 
 def bip39_to_seed(mnemonic, passphrase):
-    import pbkdf2, hashlib, hmac
+    import hashlib, hmac
     PBKDF2_ROUNDS = 2048
     mnemonic = normalize('NFKD', ' '.join(mnemonic.split()))
     passphrase = bip39_normalize_passphrase(passphrase)
-    return pbkdf2.PBKDF2(mnemonic, 'mnemonic' + passphrase,
-                         iterations = PBKDF2_ROUNDS, macmodule = hmac,
-                         digestmodule = hashlib.sha512).read(64)
+    return hashlib.pbkdf2_hmac('sha512', mnemonic.encode('utf-8'),
+        b'mnemonic' + passphrase.encode('utf-8'), iterations = PBKDF2_ROUNDS)
 
 # returns tuple (is_checksum_valid, is_wordlist_valid)
 def bip39_is_checksum_valid(mnemonic):
@@ -666,7 +664,7 @@ def hardware_keystore(d):
     if hw_type in hw_keystores:
         constructor = hw_keystores[hw_type]
         return constructor(d)
-    raise WalletFileException('unknown hardware type: {}'.format(hw_type))
+    raise WalletFileException('unknown hardware type: {}. hw_keystores: {}'.format(hw_type, list(hw_keystores.keys())))
 
 def load_keystore(storage, name):
     d = storage.get(name, {})
