@@ -47,6 +47,7 @@ from .interface import Connection, Interface
 from . import blockchain
 from .version import ELECTRUM_VERSION, PROTOCOL_VERSION
 from .i18n import _
+from .blockchain import InvalidHeader
 
 
 NODES_RETRY_INTERVAL = 60
@@ -78,6 +79,7 @@ def parse_servers(result):
             servers[host] = out
     return servers
 
+
 def filter_version(servers):
     def is_recent(version):
         try:
@@ -87,7 +89,11 @@ def filter_version(servers):
     return {k: v for k, v in servers.items() if is_recent(v.get('version'))}
 
 
-def filter_protocol(hostmap, protocol = 's'):
+def filter_noonion(servers):
+    return {k: v for k, v in servers.items() if not k.endswith('.onion')}
+
+
+def filter_protocol(hostmap, protocol='s'):
     '''Filters the hostmap for those implementing protocol.
     The result is a list in serialized form.'''
     eligible = []
@@ -97,12 +103,14 @@ def filter_protocol(hostmap, protocol = 's'):
             eligible.append(serialize_server(host, port, protocol))
     return eligible
 
+
 def pick_random_server(hostmap = None, protocol = 's', exclude_set = set()):
     if hostmap is None:
         hostmap = constants.net.DEFAULT_SERVERS
     eligible = list(set(filter_protocol(hostmap, protocol)) - exclude_set)
     return random.choice(eligible) if eligible else None
 
+
 from .simple_config import SimpleConfig
 
 proxy_modes = ['socks4', 'socks5', 'http']
@@ -166,6 +174,7 @@ class Network(util.DaemonThread):
           get_parameters(), get_server_height(), get_status_value(),
           is_connected(), set_parameters(), stop()
     """
+    verbosity_filter = 'n'
 
     def __init__(self, config=None):
         if config is None:
@@ -201,7 +210,7 @@ class Network(util.DaemonThread):
         self.pending_sends = []
         self.message_id = 0
         self.debug = False
-        self.irc_servers = {} # returned by interface (list from irc)
+        self.irc_servers = {}  # returned by interface (list from irc)
         self.recent_servers = self.read_recent_servers()  # note: needs self.recent_servers_lock
 
         self.banner = ''
@@ -213,7 +222,7 @@ class Network(util.DaemonThread):
         # callbacks set by the GUI
         self.callbacks = defaultdict(list)      # note: needs self.callback_lock
 
-        dir_path = os.path.join( self.config.path, 'certs')
+        dir_path = os.path.join(self.config.path, 'certs')
         util.make_dir(dir_path)
 
         # subscriptions and requests
@@ -312,9 +321,6 @@ class Network(util.DaemonThread):
     def is_connecting(self):
         return self.connection_status == 'connecting'
 
-    def is_up_to_date(self):
-        return self.unanswered_requests == {}
-
     @with_interface_lock
     def queue_request(self, method, params, interface=None):
         # If you want to queue a request on any interface it must go
@@ -406,7 +412,9 @@ class Network(util.DaemonThread):
                 except:
                     continue
                 if host not in out:
-                    out[host] = { protocol:port }
+                    out[host] = {protocol: port}
+        if self.config.get('noonion'):
+            out = filter_noonion(out)
         return out
 
     @with_interface_lock
@@ -416,7 +424,7 @@ class Network(util.DaemonThread):
                 self.print_error("connecting to %s as new interface" % server)
                 self.set_status('connecting')
             self.connecting.add(server)
-            c = Connection(server, self.socket_queue, self.config.path)
+            Connection(server, self.socket_queue, self.config.path)
 
     def start_random_interface(self):
         with self.interface_lock:
@@ -529,9 +537,10 @@ class Network(util.DaemonThread):
         self.auto_connect = auto_connect
         if self.proxy != proxy or self.protocol != protocol:
             # Restart the network defaulting to the given server
-            self.stop_network()
-            self.default_server = server
-            self.start_network(protocol, proxy)
+            with self.interface_lock:
+                self.stop_network()
+                self.default_server = server
+                self.start_network(protocol, proxy)
         elif self.default_server != server:
             self.switch_to_interface(server)
         else:
@@ -552,7 +561,7 @@ class Network(util.DaemonThread):
         if self.server_is_lagging() and self.auto_connect:
             # switch to one that has the correct header (not height)
             header = self.blockchain().read_header(self.get_local_height())
-            filtered = list(map(lambda x:x[0], filter(lambda x: x[1].tip_header==header, self.interfaces.items())))
+            filtered = list(map(lambda x: x[0], filter(lambda x: x[1].tip_header == header, self.interfaces.items())))
             if filtered:
                 choice = random.choice(filtered)
                 self.switch_to_interface(choice)
@@ -568,6 +577,7 @@ class Network(util.DaemonThread):
             self.interface = None
             self.start_interface(server)
             return
+
         i = self.interfaces[server]
         if self.interface != i:
             self.print_error("switching to", server)
@@ -600,7 +610,7 @@ class Network(util.DaemonThread):
 
     def process_response(self, interface, response, callbacks):
         if self.debug:
-            self.print_error("<--", response)
+            self.print_error(interface.host, "<--", response)
         error = response.get('error')
         result = response.get('result')
         method = response.get('method')
@@ -797,6 +807,7 @@ class Network(util.DaemonThread):
             server, socket = self.socket_queue.get()
             if server in self.connecting:
                 self.connecting.remove(server)
+
             if socket:
                 self.new_interface(server, socket)
             else:
@@ -871,13 +882,17 @@ class Network(util.DaemonThread):
         if not connect:
             self.connection_down(interface.server)
             return
-        # If not finished, get the next chunk
-        if index >= len(blockchain.checkpoints) and blockchain.height() < interface.tip:
-            self.request_chunk(interface, index+1)
+        if index >= len(blockchain.checkpoints):
+            # If not finished, get the next chunk
+            if blockchain.height() < interface.tip:
+                self.request_chunk(interface, index+1)
+            else:
+                interface.mode = 'default'
+                interface.print_error('catch up done', blockchain.height())
+                blockchain.catch_up = None
         else:
-            interface.mode = 'default'
-            interface.print_error('catch up done', blockchain.height())
-            blockchain.catch_up = None
+            # the verifier must have asked for this chunk
+            pass
         self.notify('updated')
 
     def on_get_header(self, interface, response):
@@ -888,6 +903,7 @@ class Network(util.DaemonThread):
             self.connection_down(interface.server)
             return
         height = header.get('block_height')
+        #interface.print_error('got header', height, blockchain.hash_header(header))
         if interface.request != height:
             interface.print_error("unsolicited header",interface.request, height)
             self.connection_down(interface.server)
@@ -902,6 +918,9 @@ class Network(util.DaemonThread):
                 next_height = height + 1
                 interface.blockchain.catch_up = interface.server
             elif chain:
+                # FIXME should await "initial chunk download".
+                # binary search will NOT do the correct thing if we don't yet
+                # have all headers up to the fork height
                 interface.print_error("binary search")
                 interface.mode = 'binary'
                 interface.blockchain = chain
@@ -943,9 +962,9 @@ class Network(util.DaemonThread):
                     elif branch.parent().check_header(header):
                         interface.print_error('reorg', interface.bad, interface.tip)
                         interface.blockchain = branch.parent()
-                        next_height = None
+                        next_height = interface.bad
                     else:
-                        interface.print_error('checkpoint conflicts with existing fork', branch.path())
+                        interface.print_error('forkpoint conflicts with existing fork', branch.path())
                         branch.write(b'', 0)
                         branch.save_header(interface.bad_header)
                         interface.mode = 'catch_up'
@@ -961,10 +980,12 @@ class Network(util.DaemonThread):
                             with self.blockchains_lock:
                                 self.blockchains[interface.bad] = b
                             interface.blockchain = b
-                            interface.print_error("new chain", b.checkpoint)
+                            interface.print_error("new chain", b.forkpoint)
                             interface.mode = 'catch_up'
-                            next_height = interface.bad + 1
-                            interface.blockchain.catch_up = interface.server
+                            maybe_next_height = interface.bad + 1
+                            if maybe_next_height <= interface.tip:
+                                next_height = maybe_next_height
+                                interface.blockchain.catch_up = interface.server
                     else:
                         assert bh == interface.good
                         if interface.blockchain.catch_up is None and bh < interface.tip:
@@ -999,14 +1020,18 @@ class Network(util.DaemonThread):
             raise Exception(interface.mode)
         # If not finished, get the next header
         if next_height is not None:
-            if interface.mode == 'catch_up' and interface.tip > next_height + 50:
+            if next_height < 0:
+                self.connection_down(interface.server)
+                next_height = None
+            elif interface.mode == 'catch_up' and interface.tip > next_height + 50:
                 self.request_chunk(interface, next_height // 2016)
             else:
                 self.request_header(interface, next_height)
-        else:
+        if next_height is None:
             interface.mode = 'default'
             interface.request = None
             self.notify('updated')
+
         # refresh network dialog
         self.notify('interfaces')
 
@@ -1065,8 +1090,18 @@ class Network(util.DaemonThread):
         self.on_stop()
 
     def on_notify_header(self, interface, header_dict):
-        header_hex, height = header_dict['hex'], header_dict['height']
-        header = blockchain.deserialize_header(util.bfh(header_hex), height)
+        try:
+            header_hex, height = header_dict['hex'], header_dict['height']
+        except KeyError:
+            # no point in keeping this connection without headers sub
+            self.connection_down(interface.server)
+            return
+        try:
+            header = blockchain.deserialize_header(util.bfh(header_hex), height)
+        except InvalidHeader:
+            self.connection_down(interface.server)
+            return
+        #interface.print_error('notified of header', height, blockchain.hash_header(header))
         if height < self.max_checkpoint():
             self.connection_down(interface.server)
             return
@@ -1111,7 +1146,7 @@ class Network(util.DaemonThread):
     @with_interface_lock
     def blockchain(self):
         if self.interface and self.interface.blockchain is not None:
-            self.blockchain_index = self.interface.blockchain.checkpoint
+            self.blockchain_index = self.interface.blockchain.forkpoint
         return self.blockchains[self.blockchain_index]
 
     @with_interface_lock
