@@ -35,17 +35,14 @@ import urllib.parse
 try:
     from . import paymentrequest_pb2 as pb2
 except ImportError:
-    sys.exit("Error: could not find paymentrequest_pb2.py. Create it with 'protoc --proto_path=lib/ --python_out=lib/ lib/paymentrequest.proto'")
+    sys.exit("Error: could not find paymentrequest_pb2.py. Create it with 'protoc --proto_path=electrum/ --python_out=electrum/ electrum/paymentrequest.proto'")
 
-from . import bitcoin
-from . import util
+from . import bitcoin, ecc, util, transaction, x509, rsakey
 from .util import print_error, bh2u, bfh
 from .util import export_meta, import_meta
-from . import transaction
-from . import x509
-from . import rsakey
 
 from .bitcoin import TYPE_ADDRESS
+from .transaction import TxOutput
 
 REQUEST_HEADERS = {'Accept': 'application/bitcoin-paymentrequest', 'User-Agent': 'Electrum'}
 ACK_HEADERS = {'Content-Type':'application/bitcoin-payment','Accept':'application/bitcoin-paymentack','User-Agent':'Electrum'}
@@ -95,7 +92,8 @@ def get_payment_request(url):
             data = None
             error = "payment URL not pointing to a valid file"
     else:
-        raise Exception("unknown scheme", url)
+        data = None
+        error = "Unknown scheme for payment request. URL: {}".format(url)
     pr = PaymentRequest(data, error)
     return pr
 
@@ -110,7 +108,7 @@ class PaymentRequest:
         self.tx = None
 
     def __str__(self):
-        return self.raw
+        return str(self.raw)
 
     def parse(self, r):
         if self.error:
@@ -127,7 +125,7 @@ class PaymentRequest:
         self.outputs = []
         for o in self.details.outputs:
             addr = transaction.get_address_from_output_script(o.script)[1]
-            self.outputs.append((TYPE_ADDRESS, addr, o.amount))
+            self.outputs.append(TxOutput(TYPE_ADDRESS, addr, o.amount))
         self.memo = self.details.memo
         self.payment_url = self.details.payment_url
 
@@ -206,9 +204,9 @@ class PaymentRequest:
         if pr.pki_type == "dnssec+btc":
             self.requestor = alias
             address = info.get('address')
-            pr.signature = ''
+            pr.signature = b''
             message = pr.SerializeToString()
-            if bitcoin.verify_message(address, sig, message):
+            if ecc.verify_message_with_address(address, sig, message):
                 self.error = 'Verified with DNSSEC'
                 return True
             else:
@@ -229,8 +227,8 @@ class PaymentRequest:
 
     def get_address(self):
         o = self.outputs[0]
-        assert o[0] == TYPE_ADDRESS
-        return o[1]
+        assert o.type == TYPE_ADDRESS
+        return o.address
 
     def get_requestor(self):
         return self.requestor if self.requestor else self.get_address()
@@ -321,10 +319,9 @@ def sign_request_with_alias(pr, alias, alias_privkey):
     pr.pki_type = 'dnssec+btc'
     pr.pki_data = str(alias)
     message = pr.SerializeToString()
-    ec_key = bitcoin.regenerate_key(alias_privkey)
-    address = bitcoin.address_from_private_key(alias_privkey)
+    ec_key = ecc.ECPrivkey(alias_privkey)
     compressed = bitcoin.is_compressed(alias_privkey)
-    pr.signature = ec_key.sign_message(message, compressed, address)
+    pr.signature = ec_key.sign_message(message, compressed)
 
 
 def verify_cert_chain(chain):
