@@ -4,7 +4,8 @@ import time
 import os
 import stat
 from decimal import Decimal
-from typing import Union
+from typing import Union, Optional
+from numbers import Real
 
 from copy import deepcopy
 
@@ -37,7 +38,7 @@ def set_config(c):
     config = c
 
 
-FINAL_CONFIG_VERSION = 2
+FINAL_CONFIG_VERSION = 3
 
 
 class SimpleConfig(PrintError):
@@ -164,6 +165,7 @@ class SimpleConfig(PrintError):
             self.print_error('upgrading config')
 
             self.convert_version_2()
+            self.convert_version_3()
 
             self.set_key('config_version', FINAL_CONFIG_VERSION, save=True)
 
@@ -186,6 +188,19 @@ class SimpleConfig(PrintError):
 
         self.set_key('config_version', 2)
 
+    def convert_version_3(self):
+        if not self._is_upgrade_method_needed(2, 2):
+            return
+
+        base_unit = self.user_config.get('base_unit')
+        if isinstance(base_unit, str):
+            self._set_key_in_user_config('base_unit', None)
+            map_ = {'btc':8, 'mbtc':5, 'ubtc':2, 'bits':2, 'sat':0}
+            decimal_point = map_.get(base_unit.lower())
+            self._set_key_in_user_config('decimal_point', decimal_point)
+
+        self.set_key('config_version', 3)
+
     def _is_upgrade_method_needed(self, min_version, max_version):
         cur_version = self.get_config_version()
         if cur_version > max_version:
@@ -281,35 +296,51 @@ class SimpleConfig(PrintError):
             return fee
         return get_fee_within_limits
 
-    @impose_hard_limits_on_fee
-    def eta_to_fee(self, slider_pos) -> Union[int, None]:
+    def eta_to_fee(self, slider_pos) -> Optional[int]:
         """Returns fee in sat/kbyte."""
         slider_pos = max(slider_pos, 0)
         slider_pos = min(slider_pos, len(FEE_ETA_TARGETS))
         if slider_pos < len(FEE_ETA_TARGETS):
-            target_blocks = FEE_ETA_TARGETS[slider_pos]
-            fee = self.fee_estimates.get(target_blocks)
+            num_blocks = FEE_ETA_TARGETS[slider_pos]
+            fee = self.eta_target_to_fee(num_blocks)
         else:
-            fee = self.fee_estimates.get(2)
-            if fee is not None:
-                fee += fee/2
-                fee = int(fee)
+            fee = self.eta_target_to_fee(1)
         return fee
 
-    def fee_to_depth(self, target_fee):
+    @impose_hard_limits_on_fee
+    def eta_target_to_fee(self, num_blocks: int) -> Optional[int]:
+        """Returns fee in sat/kbyte."""
+        if num_blocks == 1:
+            fee = self.fee_estimates.get(2)
+            if fee is not None:
+                fee += fee / 2
+                fee = int(fee)
+        else:
+            fee = self.fee_estimates.get(num_blocks)
+        return fee
+
+    def fee_to_depth(self, target_fee: Real) -> int:
+        """For a given sat/vbyte fee, returns an estimate of how deep
+        it would be in the current mempool in vbytes.
+        Pessimistic == overestimates the depth.
+        """
         depth = 0
         for fee, s in self.mempool_fees:
             depth += s
             if fee <= target_fee:
                 break
-        else:
-            return 0
         return depth
 
-    @impose_hard_limits_on_fee
     def depth_to_fee(self, slider_pos) -> int:
         """Returns fee in sat/kbyte."""
         target = self.depth_target(slider_pos)
+        return self.depth_target_to_fee(target)
+
+    @impose_hard_limits_on_fee
+    def depth_target_to_fee(self, target: int) -> int:
+        """Returns fee in sat/kbyte.
+        target: desired mempool depth in vbytes
+        """
         depth = 0
         for fee, s in self.mempool_fees:
             depth += s
@@ -317,6 +348,10 @@ class SimpleConfig(PrintError):
                 break
         else:
             return 0
+        # add one sat/byte as currently that is
+        # the max precision of the histogram
+        fee += 1
+        # convert to sat/kbyte
         return fee * 1000
 
     def depth_target(self, slider_pos):
